<?php

/*

pmPDF - Simple PHP class to create 'pocketmod' style booklets
http://www.aaronland.info/php/pmPDF/

Copyright (c) 2007 Aaron Straup Cope.

This is free software, you may use it and distribute it under the
same terms as Perl itself.

$Id: pmPDF.php,v 1.13 2007/02/03 04:52:05 asc Exp $

*/

class pmPDF extends FPDF {

		function pmPDF($args=array()) {

			if (! is_array($args['dimensions'])){
				$args['dimensions'] = array(8.5, 11);
			}

			# 

			$this->_pg   = 1;
			$this->_sect = null;
			
			$this->_pages = array(
					      1 => 'c',
					      2 => 'e',
					      3 => 'g',
					      4 => 'h',
					      5 => 'f',
					      6 => 'd',
					      7 => 'b',
					      8 => 'a',
					      );
			
			$this->_types = array ('a' => array('type' => 'D', 'y' => .25),
					       'b' => array('type' => 'U', 'y' => 2.5),
					       'c' => array('type' => 'D', 'y' => 3),
					       'd' => array('type' => 'U', 'y' => 5.25), 
					       'e' => array('type' => 'D', 'y' => 5.75),
					       'f' => array('type' => 'U', 'y' => 8),
					       'g' => array('type' => 'D', 'y' => 8.5),
					       'h' => array('type' => 'U', 'y' => 10.75));

			$this->_buffer = array();

			$this->_page_dims   = $args['dimensions'];
			$this->_sect_width  = 2.25;
			$this->_sect_height = 3.5625;

			$this->_dpi = 72;

			$this->_lastx = null;
			$this->_lasty = null;

			#

			$this->_args = $args;

			$this->FPDF("P", "in", $this->_page_dims);			
			$this->Open();

		}

		/* #################################################### */

		function position_image($path, $pg){

			$sect = $this->sect_for_page_number($pg);
			$type = $this->_pages[$sect];
			$data = $this->_types[$type];

			list($iw, $ih) = getimagesize($path);

			$iw = $iw / $this->_dpi;
			$ih = $ih / $this->_dpi;

			$start_y = $data['y'];
			$start_x = $this->start_x($data['type']);

			if ($data['type']=='D'){
				$start_x -= $iw;
			}

			else {
				$start_y -= $ih;
			}

			$end_y = $start_y + $this->_sect_height;
			$end_x = $start_x + $this->_sect_width;

			# check to see if the offset of the
			# image will bleed over a 'page' - resize
			# it if does...

			$this->Image($path, $start_x, $start_y);
		}

		/* #################################################### */

		function add_image($path, $pg='', $x='', $y=''){

			if (! function_exists('imagerotate')){
				$this->position_image($path, $pg);
				return;
			}

			$rotate = ($data['type']=='D') ? 90 : 270;

			$old = imagecreatefromjpeg($path);
			$new = imagerotate($old, $rotate);

			$tmp = tempnam("/tmp", time());
			imagejpeg($new, $tmp);

			$this->position_image($tmp, $pg);
			unlink($tmp);
		}

		/* #################################################### */

		function add_text($txt, $pg=''){

			$resume = true;

			if (! $pg){
				$pg = $this->_pg;
			}

			else {
				if ($pg != $this->_pg){
					$resume = false;
				}
			}

			# echo "RESUME ... {$pg} - {$this->_pg} ({$resume})\n";

			$this->buffer($txt, $pg, $resume);

			return array(
				     'page' => $this->_pg,
				     'x' => $this->_lastx,
				     'y' => $this->_lasty,
				     );
		}

		/* #################################################### */

		function Output($name='', $dest=''){

			$keys = array_keys($this->_buffer);
			rsort($keys);

			$cnt    = array_shift($keys);
			$sheets = ceil(($cnt + 1) / 8);

			$first = 1;
			$last  = $sheets * 8;

			foreach (range(1, $sheets) as $sh){

				$this->AddPage();

				if (intval($this->_args['folds'])){
					$this->draw_folds();
				}

				if (intval($this->_args['borders'])){
					$this->draw_borders();
				}

				foreach (range($first, ($first + 3)) as $i) {
					$this->write_page($i);
				}
				
				foreach (range(($last-3), $last) as $j) {
					$this->write_page($j);
				}
				
				$first += 4;
				$last -= 4;
			}

			parent::Output($name, $dest);
		}

		/* #################################################### */

		function draw_folds($gr=173){

			list($w, $h) = $this->_page_dims;

			$this->SetLineWidth(0.01);
			$this->SetDrawColor($gr, $gr, $gr);
			$this->Line(($w/2), 0, ($w/2), $h);
			
			$d = $h / 4;
			$y = $d;
			
			while ($y < $h) {
				$this->Line(0, $y, $w, $y);
				$y += $d;
			}

		}

		/* #################################################### */

		function draw_borders(){

			list($w, $h) = $this->_page_dims;

			foreach ($this->_types as $type => $data){
				$this->Line(0, $data['y'], $w, $data['y']);
			}

			# FIX ME...make me abstract
			$x = array(.25, .4375, 4, 4.5, 8.0625, 8.25);

			foreach ($x as $i){
				$this->Line($i, 0, $i, $h);
			}
		}

		/* #################################################### */

		function sect_for_page_number($pg){
			while ($pg > 8){
				$pg -= 8;
			}

			return $pg;
		}

		/* #################################################### */

		function reset_buffer(){
			$this->_buffer = array();
		}

		/* #################################################### */

		function buffer($txt, $pg, $resume=1){

			$sect = $this->sect_for_page_number($pg);

			$this->_pg   = $pg;
			$this->_sect = $sect;

			$txt = $this->buffer_do($txt, $resume);
			
			if (strlen($txt)){
				return $this->buffer($txt, ($pg + 1), $resume);
			}
			
			return 1;
		}

		/* #################################################### */

		function write_page($i){

			$this->write_page_number($i);

			$idx = $i - 1;

			if (is_array($this->_buffer[$idx])){
				$this->_out(implode("\n", $this->_buffer[$idx]));
			}
		}

		/* #################################################### */

		function write_page_number($pg){

			$sect = $this->sect_for_page_number($pg);
			$type = $this->_pages[$sect];
			$data = $this->_types[$type];

			$cf = $this->CurrentFont;

			$fa = $this->FontFamily;
			$st = $this->FontStyle;
			$pt = $this->FontSizePt;
			$sz = $this->FontSize;

			$this->SetFont('Helvetica', '', 40);
			$this->SetFontSize(10);

			$ln =  $this->FontSize;

			$w = $this->_sect_width;
			$s = $this->GetStringWidth($pg);

			$odd = ($pg % 2) ? 1 : 0;

			if ($data['type'] == 'D'){
				$x = 0.25;
				$y = ($odd) ? $data['y'] + ($w - $s) : $data['y'];
			}

			else {
				$x = 8.25;		
				$y = ($odd) ? $data['y'] - ($w - $s) : $data['y'];		
			}

			$ps = $this->TextWithDirection($x, $y, $pg, $data['type']);
			$this->_out($ps);

			$this->SetFont($fa, $st, $pt);
			$this->SetFontSize($pt);
			return;
		}

		/* #################################################### */

		function buffer_do($txt, $resume=1){

			/* Shamelessy pilfered from the FPDF Multi-Cell function */

			$type = $this->_pages[$this->_sect];
			$data = $this->_types[$type];
			
			if ($resume && $this->_lastx && $this->_lasty){
				$x = $this->_lastx;
				$y = $this->_lasty;
			}

			else {
				$y = $data['y'];
				$x =  $this->start_x($data['type']);
			}

			$w = $this->_sect_width;
			$h = $this->_sect_height;
			
			$cw = &$this->CurrentFont['cw'];

			$wmax = ($w-2*$this->cMargin)*1000/$this->FontSize;
			
			$s = str_replace("\r",'',$txt);
			
			$nb = strlen($s);
			
			if ($nb>0 && $s[$nb-1]=="\n"){
				$nb--;
			}

			$sep = -1;
			$i = 0;
			$j = 0;
			$l = 0;
			$ns = 0;
			$nl = 1;

			while ($i < $nb){
			
				if (! $this->test_x($x)) {
					$this->_lastx = null;
					$this->_lasty = null;
					return substr($s, ($j + ($i-$j)));
				}

				//Get next character
				$c = $s{$i};

				if ($c=="\n"){
					//Explicit line break
					
					if ($this->ws>0){
						$this->ws=0;

						$this->record('0 Tw');
					}
					
					$chars = substr($s, $j, $i-$j);

					$ps = $this->TextWithDirection($x, $y, $chars, $data['type']);
					$this->record($ps);

					$x = $this->calc_x($x);
					
					$i++;
					$sep=-1;
					$j=$i;
					$l=0;
					$ns=0;
					$nl++;
					continue;
				}
				
				if ($c==' '){
					$sep=$i;
					$ls=$l;
					$ns++;
				}

				$l += $cw[$c];

				if($l > $wmax){
					
					//Automatic line break
					if ($sep==-1){
						
						if ($i==$j) {
							$i++;
						}
						
						if ($this->ws>0) {
							$this->ws=0;
							$this->record('0 Tw');
						}

						$chars = substr($s, $j, $i-$j);

						$ps = $this->TextWithDirection($x, $y, $chars, $data['type']);
						$this->record($ps);

						$x = $this->calc_x($x);
					}

					else {

						$chars = substr($s,$j,$sep-$j);

						$ps = $this->TextWithDirection($x, $y, $chars, $data['type']);
						$this->record($ps);

						$x = $this->calc_x($x);
						$i=$sep+1;
					}
					
					$sep=-1;
					$j =$i;
					$l =0;
					$ns =0;
					$nl++;
					
				}
				
				else {
					$i++;
				}
			}
			
			//Last chunk
			
			if($this->ws>0) {
				$this->ws=0;
				$this->record('0 Tw');
			}
			
			$chars = substr($s, $j, $i-$j);

			$ps = $this->TextWithDirection($x, $y, $chars, $data['type']);
			$this->record($ps);

			$x = $this->calc_x($x);

			$this->_lastx = $x;
			$this->_lasty = $y;
			
			return '';
		}

		/* #################################################### */

		function record($ps){

			$idx = $this->_pg - 1;

			if (! is_array($this->_buffer[$idx])){
				$this->_buffer[$idx] = array();
			}

			$this->_buffer[$idx][] = $ps;
		}

		/* #################################################### */

		function start_x($type){

			$ln = $this->FontSize;
			$c = $this->_page_dims[0] / 2;
			$o = .25;

			$x =  ($type == 'D') ? ($c - $o - $ln)  : ($c + $o + $ln);		       
			return $x;
		}

		/* #################################################### */

		function calc_x($x){

			$type = $this->_pages[$this->_sect];
			$data = $this->_types[$type];
			
			$ln =  $this->FontSize * 1.25;
			
			if ($data['type']=='U'){
				$x += $ln;
				return $x;
			}
			
			$x -= $ln;
			return $x;
		}

		/* #################################################### */
		
		function test_x ($x){
			
			$type = $this->_pages[$this->_sect];
			$data = $this->_types[$type];

			#  FIX ME...make abstract

			if ($data['type']=='U'){
				$ok = ($x < 8.0625) ? 1 : 0;
			}
			
			else {
				$ok = ($x > .4375) ? 1 : 0;
			}
			
			return $ok;
		}

		/* #################################################### */

		function TextWithDirection($x,$y,$txt,$direction='D') {

			/* Shamelessly pilfered from the RPDF TextWithDirection function */

			$txt = str_replace(')','\\)',str_replace('(','\\(',str_replace('\\','\\\\',$txt)));

			if ($direction=='U'){
				$s=sprintf('BT %.2f %.2f %.2f %.2f %.2f %.2f Tm (%s) Tj ET',0,1,-1,0,$x*$this->k,($this->h-$y)*$this->k,$txt);
			}

			else {
				$s=sprintf('BT %.2f %.2f %.2f %.2f %.2f %.2f Tm (%s) Tj ET',0,-1,1,0,$x*$this->k,($this->h-$y)*$this->k,$txt);
			}
			
			return $s;
		}

		/* #################################################### */
}

?> 
